/*
 * File:	JarLoader.java
 * Author:	Dmitry Rogatkin (based on Sun's code)
 * Date:	03/11/98
 *
 * Description:	JarLoader
 * Read the contents of a JAR file
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.io;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.beans.*;
import java.net.*;
import java.awt.*;

import rogatkin.xbox.shared.DoOnBean;

public class JarLoader {
	
    private static boolean debug = false; // debugging
    private InputStream jarStream; // Jar input stream
    private String jarName;	// name of the jar file
    private SimpleClassLoader loader; // the loader instance
    private static boolean warnedAboutNoBeans;
    private static boolean defineOnDemand = true;
	
	
    /**
	* Create a JarLoader to read a JAR and to process its contents.
	* Classes and resources are loaded against a single common class
	* loader instance so that things like "adaptor class instantiaton"
	* can work.
	*
	* Loading is started with loadIt()
	*/
    public JarLoader(String jarName) throws FileNotFoundException {
		// will check that this file exists, and that is about it.
		debug("("+jarName+")");
		this.jarName = jarName;
		InputStream is = new FileInputStream(jarName);
		jarStream = new BufferedInputStream(is);
		loader = SimpleClassLoader.ourLoader;
    }

    public JarLoader(URL jarUrl) throws IOException {
		debug("("+jarUrl+")");
		this.jarName = jarUrl.toString();
		jarStream = new BufferedInputStream(jarUrl.openStream());
		loader = SimpleClassLoader.ourLoader;
    }
	
    /**
	* get the loader we are using
	*/
    public ClassLoader getLoader() {
		return loader;
    }
	
    public static boolean getDefineOnDemand() {
		return defineOnDemand;
    }

	/*
	* In here for compatibility with older versions of JDK1.1
	*/
    private String guessContentTypeFromStream(InputStream is) throws IOException {
		String type;
		type = URLConnection.guessContentTypeFromStream(is);
		// that should be taught about serialized objects.
		
		if (type == null) {
			is.mark(10);
			int c1 = is.read();
			int c2 = is.read();
			int c3 = is.read();
			int c4 = is.read();
			int c5 = is.read();
			int c6 = is.read();
			is.reset();
			if (c1 == 0xAC && c2 == 0xED) {
				type = "application/java-serialized-object";
			}
		}
		return type;
    }
	
    /**
	* Load the classes, resources, etc.
	*/
    public JarInfo loadJar() {
		// load all resources.
		// may raise an exception if the file is not a Jar file.
		ZipInputStream zis = null;
		Manifest mf = null;
		Vector classList = new Vector(); // the classes
		Vector serList = new Vector(); // the serialized objects
		
		byte buffer[] = new byte[1024];
		
		try {
			zis = new ZipInputStream(jarStream);
			ZipEntry ent = null;
			
			while ((ent = zis.getNextEntry()) != null) {
				
				String name = ent.getName();
				String type = null;
				
				/* the object we're loading */
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				
				/* NOTE: We don't know the size of an entry until
				we've reached the end of one because of
				compression. This means we can't just do a get size
				and read in the entry. */
				
				for (;;) {
					int len = zis.read(buffer);
					if (len < 0) {
						break;
					}
					baos.write(buffer, 0, len);
				}
				
				byte[] buf = baos.toByteArray();
				int size = buf.length;
				
				if (Manifest.isManifestName(name)) {
					type = "manifest/manifest";
				}
				
				if (type == null) {
					InputStream tmpStream = new ByteArrayInputStream(buf);
					type = guessContentTypeFromStream(tmpStream);
					tmpStream.close();
				}
				
				if (type == null) {
					type = "input-stream/input-stream";
				}
				
				if (type.startsWith("application/java-serialized-object") ||
					type.startsWith("application/x-java-serialized-object")) {
					// Else make the data available as a local stream.
					loader.putLocalResource(name, buf, type);
					
					// Tag it as a serialized object for the bean list.
					String sername = name.substring(0, name.length() - 4);
					sername = sername.replace('/', '.');
					serList.addElement(sername);
					
				} else if (type.startsWith("application/java-vm") ||
					type.startsWith("application/x-java-vm")) {
					/*
					* In JDK1.1, a classloader needs not provide access to the
					* bytecodes of a .class resource.  But it needs to provide
					* a URL for it (e.g. for URL math computation).
					*
					* The bytecodes are kept (and later released) by the
					* SimpleClassLoader instance.
					* Here we just tag the existance of the object.
					*/
					
					loader.putClassResource(name, type);
					
					/* remove the .class suffix */
					String classname = name.substring(0, name.length() - 6);
					classname = classname.replace('/', '.');
					loader.defineClassFromBytes(classname, buf);
					classList.addElement(classname);
					
				} else if (type.equals("manifest/manifest")) {
					mf = new Manifest(buf);
					
				} else {
					// Else make the data available as a local stream.
					loader.putLocalResource(name, buf, type);
				}
			}
			
		} catch (IOException e) {
			debug("IOException loading archive: " + e);
			e.printStackTrace();
		} catch (Throwable ex) {
			debug("Caught "+ex+" in loadit()");
			ex.printStackTrace();
		} finally {
			if (zis != null) {
				try {
					zis.close();
				} catch (Exception ex) {
					// ignore
				}
			}
		}
		
		if (!getDefineOnDemand()) {
			if (! loader.applyDefinitions(classList)) {
				return null;
			}
		}
		JarInfo ji = createJarInfo(classList, serList, mf);
		return ji;
	}

	public static void loadJarDoOnBean(String jarFile, DoOnBean action) {
	    loadJarDoOnBean((Object)jarFile, action);
	}

	public static void loadJarDoOnBean(URL jarURL, DoOnBean action) {
	    loadJarDoOnBean((Object)jarURL, action);
	}

	/**
	* Load the JAR file, then apply an action to each bean found
	*/
	private static void loadJarDoOnBean(Object jarLoc, DoOnBean action) {
	    JarLoader jl = null;
	    try {
		if (jarLoc instanceof String)
		    jl = new JarLoader((String)jarLoc);
		else if (jarLoc instanceof URL)
		    jl = new JarLoader((URL)jarLoc);
		if (jl == null) {
		    System.err.println("file name passed is not a valid JarFile");
		    return;
		}
	    } catch (Exception ex) {
		System.err.println("caught "+ex+" while trying to load "+jarLoc);
		ex.printStackTrace();
		return;
	    }
	    try {
		JarInfo ji = jl.loadJar();
		if (ji == null) {
		    System.err.println("JAR file "+jarLoc+" did not load properly!");
		    System.err.println("Check for error messages possibly regarding");
		    System.err.println("problems defining classes");
		    return;
		}
		if (ji.getCount() == 0) {
		    System.err.println("JAR file "+jarLoc+" didn't have any beans!");
		    if (!warnedAboutNoBeans) {
			// We only print this explanatory message once.
			warnedAboutNoBeans = true;	
			System.err.println();
			System.err.println("Each jar file needs to contain a manifest file describing which entries are");
			System.err.println("beans.  You can should provide a suitable manifest when you create the jar.");
			System.err.println();
		    }
		}
		for (int i=0; i<ji.getCount(); i++) {
		    String beanName = ji.getName(i);
		    try {
			BeanInfo bi = ji.getBeanInfo(i);
			
			if (bi == null) {
			    // We couldn't load the bean.
			    continue;
			}
			
			action.action(ji, bi, beanName);
		    } catch (Exception e) {
			System.err.println("Can't perform an action on bean "+beanName+" from JAR "+jarLoc+" ("+e+')');
			if (debug)
			    e.printStackTrace();
		    }
		}
	    } catch (Throwable t) {
		System.err.println("Uncaught exception '"+t+"' happened at the attempt to load beans from JAR file "+jarLoc);
		if (debug)
		    t.printStackTrace();
	    }
	}


	/**
	* Create a JarInfo from a manifest and a class list
	*/

	private JarInfo createJarInfo(Vector classList,
								  Vector serList,
								  Manifest mf) {
		Hashtable beans;
		Hashtable headersTable = new Hashtable();
		if (mf == null) {
			beans = new Hashtable();
			for (Enumeration keys = classList.elements();
			keys.hasMoreElements(); ) {
				String key = (String) keys.nextElement();
				beans.put(key, new Boolean(false)); // not from serializable
			}
			for (Enumeration keys = serList.elements();
			keys.hasMoreElements(); ) {
				String key = (String) keys.nextElement();
				beans.put(key, new Boolean(true)); // from class definition
			}
		} else {
			beans = new Hashtable();
			for (Enumeration entries = mf.entries();
			entries.hasMoreElements();) {
				MessageHeader mh = (MessageHeader) entries.nextElement();
				String name = mh.findValue("Name");
				String isBean = mh.findValue("Java-Bean");
				if (isBean != null && isBean.equalsIgnoreCase("True")) {
					String beanName;
					boolean fromPrototype = true;
					if (name.endsWith(".class")) {
						fromPrototype = false;
						beanName = name.substring(0, name.length() - 6);
					} else if (name.endsWith(".ser")) {
						beanName = name.substring(0, name.length() - 4);
					} else {
						beanName = name;
					}
					beanName = beanName.replace('/', '.');
					beans.put(beanName, new Boolean(fromPrototype));
					headersTable.put(beanName, mh);
				}
			}
		}
		
		String beanNames[] = new String[beans.size()];
		boolean fromPrototype[] = new boolean[beans.size()];
		MessageHeader headers[] = new MessageHeader[beans.size()];
		Enumeration keys;
		int i;
		for (keys = beans.keys(), i = 0;
		keys.hasMoreElements();
		i++) {
			String key = (String) keys.nextElement();
			beanNames[i] = key;
			fromPrototype[i] = ((Boolean)beans.get(key)).booleanValue();
			headers[i] = (MessageHeader) headersTable.get(key);
		}
		
		return new JarInfo(jarName, loader, beanNames, fromPrototype, headers);
	}


	/**
	* Debugging stuff
	*/
	private static void debug(String msg) {
		if (debug) {
			System.err.println("JarLoader:: "+msg);
		}
	}

}
